library(sas7bdat)
library(ggplot2)
library(cowplot)
library(survival)

#Load in the two SAS datasets
weather<-read.sas7bdat('c:/Users/ToBr708/Downloads/ncdc_edmonton.sas7bdat')
weather$date<-as.Date(weather$DATE, origin='1960-01-01')
pollution<-read.sas7bdat('c:/Users/ToBr708/Downloads/naps_edmonton.sas7bdat')
pollution$date<-as.Date(pollution$DATE, origin='1960-01-01')

envir<-merge(weather, pollution)


#####
## NULL Distributions
#####

coefs.all<-numeric()
coefsc.all<-numeric()

for(i in c(1:10)){
  print(i)
  #Generate the events and referents
  period<-seq(as.Date('2000/04/01'), as.Date('2010/03/31'), by='day')
  events<-sample(period, 5000, replace=T)
  id<-1:length(events)
  
  events.df<-data.frame('date'=events, 'date0'=events, 'case'=1, 'id'=id)
  referents1.df<-data.frame('date'= events, 'date0'=events-7, 'case'=0, 'id'=id)
  referents2.df<-data.frame('date'= events, 'date0'=events-14, 'case'=0, 'id'=id)
  referents3.df<-data.frame('date'= events, 'date0'=events-21, 'case'=0, 'id'=id)
  referents4.df<-data.frame('date'= events, 'date0'=events-28, 'case'=0, 'id'=id)
  referents1a.df<-data.frame('date'= events, 'date0'=events+7, 'case'=0, 'id'=id)
  referents2a.df<-data.frame('date'= events, 'date0'=events+14, 'case'=0, 'id'=id)
  referents3a.df<-data.frame('date'= events, 'date0'=events+21, 'case'=0, 'id'=id)
  referents4a.df<-data.frame('date'= events, 'date0'=events+28, 'case'=0, 'id'=id)
  
  
  all.dates<-rbind(events.df, referents1.df, referents2.df, referents3.df, referents4.df,
                   referents1a.df, referents2a.df, referents3a.df, referents4a.df)
  all.datesTS<-all.dates[format(all.dates$date0, '%m')==format(all.dates$date, '%m') & format(all.dates$date0, '%Y')==format(all.dates$date, '%Y'),]
  
  dataset<-merge(all.datesTS, envir, by.x='date0', by.y='date')
  
  
  ##Analysis
  #logistic regression
  NO.glm<-glm(case~NO + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  CO.glm<-glm(case~CO + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  NO2.glm<-glm(case~NO2 + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  O3.glm<-glm(case~O3 + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  PM25.glm<-glm(case~PM25 + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  
  coefs<-c(coef(NO.glm)[2], coef(CO.glm)[2], coef(NO2.glm)[2], coef(O3.glm)[2], coef(PM25.glm)[2])
  names(coefs)<-c('NO', 'CO', 'NO2', 'O3', 'PM25')
  
  coefs.all<-c(coefs.all, coefs)
  
  NO.cglm<-clogit(case~NO + TEMP + DEWP + WDSP + strata(id), data=dataset)
  CO.cglm<-clogit(case~CO + TEMP + DEWP + WDSP + strata(id), data=dataset)
  NO2.cglm<-clogit(case~NO2 + TEMP + DEWP + WDSP + strata(id), data=dataset)
  O3.cglm<-clogit(case~O3 + TEMP + DEWP + WDSP + strata(id), data=dataset)
  PM25.cglm<-clogit(case~PM25 + TEMP + DEWP + WDSP + strata(id), data=dataset)
  
  coefsc<-c(coef(NO.cglm)[2], coef(CO.cglm)[2], coef(NO2.cglm)[2], coef(O3.cglm)[2], coef(PM25.cglm)[2])
  names(coefsc)<-c('NO', 'CO', 'NO2', 'O3', 'PM25')
  
  coefsc.all<-c(coefsc.all, coefsc)
  
}

log.reg<-data.frame('var'= names(coefs.all), 'coef'=coefs.all)
log.reg.agg<-aggregate(coef~var, data=log.reg, FUN=mean)
log.reg.cal<-merge(log.reg.agg, log.reg, by='var')
log.reg.cal$cal<-log.reg.cal$coef.y-log.reg.cal$coef.x
c.log.reg<-data.frame('var'= names(coefsc.all), 'coef'=coefsc.all)

##Additional code to read and write simulation runs
# write.csv(log.reg, 'c:/Users/ToBr708/Desktop/logreg3.csv')
# write.csv(log.reg.cal, 'c:/Users/ToBr708/Desktop/logregcal3.csv')
# write.csv(c.log.reg, 'c:/Users/ToBr708/Desktop/clogreg3.csv')
# 
# log.reg<-read.csv('c:/Users/ToBr708/Desktop/logreg.csv')
# log.reg.cal<-read.csv('c:/Users/ToBr708/Desktop/logregcal.csv')
# c.log.reg<-read.csv('c:/Users/ToBr708/Desktop/clogreg.csv')
# 
# log.reg2<-read.csv('c:/Users/ToBr708/Desktop/logreg2.csv')
# log.reg.cal2<-read.csv('c:/Users/ToBr708/Desktop/logregcal2.csv')
# c.log.reg2<-read.csv('c:/Users/ToBr708/Desktop/clogreg2.csv')
# 
# log.reg<-rbind(log.reg, log.reg2)
# log.reg.cal<-rbind(log.reg.cal, log.reg.cal2)
# c.log.reg<-rbind(c.log.reg, c.log.reg2)


boxplot.log.reg<-ggplot(log.reg, aes(y=coef, x=var, col=var)) + geom_boxplot() + ggtitle('logit') + geom_hline(yintercept = 0, lty='dotted')
boxplot.log.reg.cal<-ggplot(log.reg.cal, aes(y=cal, x=var, col=var)) + geom_boxplot() + ggtitle('logit.cal') + geom_hline(yintercept = 0, lty='dotted') 
boxplot.c.log.reg<-ggplot(c.log.reg, aes(y=coef, x=var, col=var)) + geom_boxplot() + ggtitle('cond.logit') + geom_hline(yintercept = 0, lty='dotted') 

null.plot<-plot_grid(boxplot.log.reg + theme(legend.position = "none"), boxplot.c.log.reg + theme(legend.position = "none"), boxplot.log.reg.cal + theme(legend.position = "none"), nrow=1, labels=c('A1', 'A2', 'A3'))
##for calibration


######
## Beta differs from zero!
######

coefsp.all<-numeric()
coefscp.all<-numeric()

for(i in c(1:10)){
  print(i)
  #Generate the events and referents
  events<-sample(envir$date, 5000, prob=exp(envir$O3*0.1), replace=T)
  id<-1:length(events)
  
  events.df<-data.frame('date'=events, 'date0'=events, 'case'=1, 'id'=id)
  referents1.df<-data.frame('date'= events, 'date0'=events-7, 'case'=0, 'id'=id)
  referents2.df<-data.frame('date'= events, 'date0'=events-14, 'case'=0, 'id'=id)
  referents3.df<-data.frame('date'= events, 'date0'=events-21, 'case'=0, 'id'=id)
  referents4.df<-data.frame('date'= events, 'date0'=events-28, 'case'=0, 'id'=id)
  referents1a.df<-data.frame('date'= events, 'date0'=events+7, 'case'=0, 'id'=id)
  referents2a.df<-data.frame('date'= events, 'date0'=events+14, 'case'=0, 'id'=id)
  referents3a.df<-data.frame('date'= events, 'date0'=events+21, 'case'=0, 'id'=id)
  referents4a.df<-data.frame('date'= events, 'date0'=events+28, 'case'=0, 'id'=id)
  
  
  all.dates<-rbind(events.df, referents1.df, referents2.df, referents3.df, referents4.df,
                   referents1a.df, referents2a.df, referents3a.df, referents4a.df)
  all.datesTS<-all.dates[format(all.dates$date0, '%m')==format(all.dates$date, '%m') & format(all.dates$date0, '%Y')==format(all.dates$date, '%Y'),]
  
  dataset<-merge(all.datesTS, envir, by.x='date0', by.y='date')
  
  
  ##Analysis
  #logistic regression
  NO.glm<-glm(case~NO + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  CO.glm<-glm(case~CO + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  NO2.glm<-glm(case~NO2 + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  O3.glm<-glm(case~O3 + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  PM25.glm<-glm(case~PM25 + TEMP + DEWP + WDSP, data=dataset, family='binomial')
  
  coefs<-c(coef(NO.glm)[2], coef(CO.glm)[2], coef(NO2.glm)[2], coef(O3.glm)[2], coef(PM25.glm)[2])
  names(coefs)<-c('NO', 'CO', 'NO2', 'O3', 'PM25')
  
  coefsp.all<-c(coefsp.all, coefs)
  
  NO.cglm<-clogit(case~NO + TEMP + DEWP + WDSP + strata(id), data=dataset)
  CO.cglm<-clogit(case~CO + TEMP + DEWP + WDSP + strata(id), data=dataset)
  NO2.cglm<-clogit(case~NO2 + TEMP + DEWP + WDSP + strata(id), data=dataset)
  O3.cglm<-clogit(case~O3 + TEMP + DEWP + WDSP + strata(id), data=dataset)
  PM25.cglm<-clogit(case~PM25 + TEMP + DEWP + WDSP + strata(id), data=dataset)
  
  coefsc<-c(coef(NO.cglm)[1], coef(CO.cglm)[1], coef(NO2.cglm)[1], coef(O3.cglm)[1], coef(PM25.cglm)[1])
  names(coefsc)<-c('NO', 'CO', 'NO2', 'O3', 'PM25')
  
  coefscp.all<-c(coefscp.all, coefsc)
  
}

log.regp<-data.frame('var'= names(coefsp.all), 'coef'=coefsp.all)
log.reg.calp<-merge(log.reg.agg, log.regp, by='var')
log.reg.calp$cal<-log.reg.calp$coef.y-log.reg.calp$coef.x
c.log.regp<-data.frame('var'= names(coefscp.all), 'coef'=coefscp.all)

##Additional code to write and read previous simulation runs
# write.csv(log.regp, 'c:/Users/ToBr708/Desktop/logregp2.csv')
# write.csv(log.reg.calp, 'c:/Users/ToBr708/Desktop/logregcalp2.csv')
# write.csv(c.log.regp, 'c:/Users/ToBr708/Desktop/clogregp2.csv')
# 
# log.regp<-read.csv('c:/Users/ToBr708/Desktop/logregp.csv')
# log.reg.calp<-read.csv('c:/Users/ToBr708/Desktop/logregcalp.csv')
# c.log.regp<-read.csv('c:/Users/ToBr708/Desktop/clogregp.csv')
# 
# log.regp2<-read.csv('c:/Users/ToBr708/Desktop/logregp2.csv')
# log.reg.calp2<-read.csv('c:/Users/ToBr708/Desktop/logregcalp2.csv')
# c.log.regp2<-read.csv('c:/Users/ToBr708/Desktop/clogregp2.csv')
# 
# log.regp<-rbind(log.regp, log.regp2)
# log.reg.calp<-rbind(log.reg.calp, log.reg.calp2)
# c.log.regp<-rbind(c.log.regp, c.log.regp2)


boxplot.log.reg.p<-ggplot(log.regp, aes(y=coef, x=var, col=var)) + geom_boxplot() + ggtitle('logit') + geom_hline(yintercept = 0, lty='dotted')  + geom_hline(yintercept = 0.1, lty='dotted', col='red')  + scale_y_continuous(limits = c(-0.1, 0.2))
boxplot.log.reg.cal.p<-ggplot(log.reg.calp, aes(y=cal, x=var, col=var)) + geom_boxplot() + ggtitle('logit.cal') + geom_hline(yintercept = 0, lty='dotted') + geom_hline(yintercept = 0.1, lty='dotted', col='red')  + scale_y_continuous(limits = c(-0.1, 0.2))
boxplot.c.log.reg.p<-ggplot(c.log.regp, aes(y=coef, x=var, col=var)) + geom_boxplot() + ggtitle('cond.logit') + geom_hline(yintercept = 0, lty='dotted') + geom_hline(yintercept = 0.1, lty='dotted', col='red')  + scale_y_continuous(limits = c(-0.1, 0.2))

beta.plot<-plot_grid(boxplot.log.reg.p + theme(legend.position = "none"), boxplot.c.log.reg.p + theme(legend.position = "none"), boxplot.log.reg.cal.p + theme(legend.position = "none"), nrow=1, labels=c('B1', 'B2', 'B3'))

plot.comparison<-plot_grid(null.plot, beta.plot, ncol=1, nrow=2)

save_plot('c:/Users/ToBr708/Desktop/biasCC.png', plot.comparison, ncol=3, nrow=2)

